@@ -39,6 +39,8 @@
 #include "database.h"
 #include "databasesize.h"
 
+#include "enginedebug.h"
+
 #include <QFile>
 #include <QFileInfo>
 
@@ -51,7 +53,10 @@ Transaction::Transaction(const Database& db, Transaction::TransactionType type)
 {
     uint flags = type == ReadOnly ? MDB_RDONLY : 0;
     int rc = mdb_txn_begin(db.m_env, nullptr, flags, &m_txn);
-    Q_ASSERT_X(rc == 0, "Transaction", mdb_strerror(rc));
+    if (rc) {
+        qCDebug(ENGINE) << "Transaction" << mdb_strerror(rc);
+        return;
+    }
 
     if (type == ReadWrite) {
         m_writeTrans = new WriteTransaction(m_dbis, m_txn);
@@ -65,8 +70,9 @@ Transaction::Transaction(Database* db, Transaction::TransactionType type)
 
 Transaction::~Transaction()
 {
-    if (m_writeTrans)
-        qWarning() << "Closing an active WriteTransaction without calling abort/commit";
+    if (m_writeTrans) {
+        qWarning(ENGINE) << "Closing an active WriteTransaction without calling abort/commit";
+    }
 
     if (m_txn) {
         abort();
@@ -95,6 +101,12 @@ bool Transaction::hasFailed(quint64 id) const
     return failedIdDb.contains(id);
 }
 
+QVector<quint64> Transaction::failedIds(quint64 limit) const
+{
+    DocumentIdDB failedIdDb(m_dbis.failedIdDbi, m_txn);
+    return failedIdDb.fetchItems(limit);
+}
+
 QByteArray Transaction::documentUrl(quint64 id) const
 {
     Q_ASSERT(m_txn);
@@ -154,7 +166,11 @@ QByteArray Transaction::documentData(quint64 id) const
 bool Transaction::hasChanges() const
 {
     Q_ASSERT(m_txn);
-    Q_ASSERT(m_writeTrans);
+    if (!m_writeTrans) {
+        qCWarning(ENGINE) << "m_writeTrans is null";
+        return false;
+    }
+
     return m_writeTrans->hasChanges();
 }
 
@@ -228,7 +244,10 @@ void Transaction::addDocument(const Document& doc)
 {
     Q_ASSERT(m_txn);
     Q_ASSERT(doc.id() > 0);
-    Q_ASSERT(m_writeTrans);
+    if (!m_writeTrans) {
+        qCWarning(ENGINE) << "m_writeTrans is null";
+        return;
+    }
 
     m_writeTrans->addDocument(doc);
 }
@@ -237,7 +256,10 @@ void Transaction::removeDocument(quint64 id)
 {
     Q_ASSERT(m_txn);
     Q_ASSERT(id > 0);
-    Q_ASSERT(m_writeTrans);
+    if (!m_writeTrans) {
+        qCWarning(ENGINE) << "m_writeTrans is null";
+        return;
+    }
 
     m_writeTrans->removeDocument(id);
 }
@@ -246,7 +268,10 @@ void Transaction::removeRecursively(quint64 id)
 {
     Q_ASSERT(m_txn);
     Q_ASSERT(id > 0);
-    Q_ASSERT(m_writeTrans);
+    if (!m_writeTrans) {
+        qCWarning(ENGINE) << "m_writeTrans is null";
+        return;
+    }
 
     m_writeTrans->removeRecursively(id);
 }
@@ -256,7 +281,14 @@ void Transaction::replaceDocument(const Document& doc, DocumentOperations operat
     Q_ASSERT(m_txn);
     Q_ASSERT(doc.id() > 0);
     Q_ASSERT(m_writeTrans);
-    Q_ASSERT_X(hasDocument(doc.id()), "Transaction::replaceDocument", "Document does not exist");
+    if (!hasDocument(doc.id())) {
+        qCDebug(ENGINE) << "Transaction::replaceDocument" << "Document does not exist";
+    }
+
+    if (!m_writeTrans) {
+        qCWarning(ENGINE) << "m_writeTrans is null";
+        return;
+    }
 
     m_writeTrans->replaceDocument(doc, operations);
 }
@@ -264,14 +296,19 @@ void Transaction::replaceDocument(const Document& doc, DocumentOperations operat
 void Transaction::commit()
 {
     Q_ASSERT(m_txn);
-    Q_ASSERT(m_writeTrans);
+    if (!m_writeTrans) {
+        qCWarning(ENGINE) << "m_writeTrans is null";
+        return;
+    }
 
     m_writeTrans->commit();
     delete m_writeTrans;
     m_writeTrans = nullptr;
 
     int rc = mdb_txn_commit(m_txn);
-    Q_ASSERT_X(rc == 0, "Transaction::commit", mdb_strerror(rc));
+    if (rc) {
+        qCWarning(ENGINE) << "Transaction::commit" << mdb_strerror(rc);
+    }
 
     m_txn = nullptr;
 }
@@ -306,24 +343,31 @@ PostingIterator* Transaction::postingIterator(const EngineQuery& query) const
         }
     }
 
-    if (query.subQueries().isEmpty()) {
+    const auto subQueries = query.subQueries();
+    if (subQueries.isEmpty()) {
         return nullptr;
     }
 
-    QVector<PostingIterator*> vec;
-    vec.reserve(query.subQueries().size());
-
     if (query.op() == EngineQuery::Phrase) {
-        const auto subQueries = query.subQueries();
+        if (subQueries.size() == 1) {
+            qCDebug(ENGINE) << "Degenerated Phrase with 1 Term:" <<  query;
+            return postingIterator(subQueries[0]);
+        }
+        QVector<VectorPositionInfoIterator*> vec;
+        vec.reserve(subQueries.size());
         for (const EngineQuery& q : subQueries) {
-            Q_ASSERT_X(q.leaf(), "Transaction::toPostingIterator", "Phrase queries must contain leaf queries");
+            if (!q.leaf()) {
+                qCDebug(ENGINE) << "Transaction::toPostingIterator" << "Phrase queries must contain leaf queries";
+                continue;
+            }
             vec << positionDb.iter(q.term());
         }
 
         return new PhraseAndIterator(vec);
     }
 
-    const auto subQueries = query.subQueries();
+    QVector<PostingIterator*> vec;
+    vec.reserve(subQueries.size());
     for (const EngineQuery& q : subQueries) {
         auto iterator = postingIterator(q);
         if (iterator) {
@@ -450,7 +494,7 @@ DatabaseSize Transaction::dbSize()
 
     dbSize.mtimeDb = dbiSize(m_txn, m_dbis.mtimeDbi);
 
-    dbSize.expectedSize = dbSize.positionDb + dbSize.positionDb + dbSize.docTerms + dbSize.docFilenameTerms
+    dbSize.expectedSize = dbSize.postingDb + dbSize.positionDb + dbSize.docTerms + dbSize.docFilenameTerms
                   + dbSize.docXattrTerms + dbSize.idTree + dbSize.idFilename + dbSize.docTime
                   + dbSize.docData + dbSize.contentIndexingIds + dbSize.failedIds + dbSize.mtimeDb;
 
@@ -472,10 +516,10 @@ void Transaction::checkFsTree()
     DocumentUrlDB docUrlDb(m_dbis.idTreeDbi, m_dbis.idFilenameDbi, m_txn);
     PostingDB postingDb(m_dbis.postingDbi, m_txn);
 
-    auto map = postingDb.toTestMap();
+    const auto map = postingDb.toTestMap();
 
     QSet<quint64> allIds;
-    Q_FOREACH (const auto& list, map) {
+    for (const auto& list : map) {
         for (quint64 id : list) {
             allIds << id;
         }
@@ -547,10 +591,10 @@ void Transaction::checkTermsDbinPostingDb()
     // Iterate over each document, and fetch all terms
     // check if each term maps to its own id in the posting db
 
-    auto map = postingDb.toTestMap();
+    const auto map = postingDb.toTestMap();
 
     QSet<quint64> allIds;
-    Q_FOREACH (const auto& list, map) {
+    for (const auto& list : map) {
         for (quint64 id : list) {
             allIds << id;
         }
