@@ -77,28 +77,26 @@ int main(int argc, char* argv[])
     parser.addPositionalArgument(QStringLiteral("status"), i18n("Print the status of the indexer"));
     parser.addPositionalArgument(QStringLiteral("enable"), i18n("Enable the file indexer"));
     parser.addPositionalArgument(QStringLiteral("disable"), i18n("Disable the file indexer"));
-    parser.addPositionalArgument(QStringLiteral("start"), i18n("Start the file indexer"));
-    parser.addPositionalArgument(QStringLiteral("stop"), i18n("Stop the file indexer"));
-    parser.addPositionalArgument(QStringLiteral("restart"), i18n("Restart the file indexer"));
+    parser.addPositionalArgument(QStringLiteral("purge"), i18n("Remove the index database"));
     parser.addPositionalArgument(QStringLiteral("suspend"), i18n("Suspend the file indexer"));
     parser.addPositionalArgument(QStringLiteral("resume"), i18n("Resume the file indexer"));
     parser.addPositionalArgument(QStringLiteral("check"), i18n("Check for any unindexed files and index them"));
-    parser.addPositionalArgument(QStringLiteral("checkDb"), i18n("Check database for consistency"));
     parser.addPositionalArgument(QStringLiteral("index"), i18n("Index the specified files"));
     parser.addPositionalArgument(QStringLiteral("clear"), i18n("Forget the specified files"));
     parser.addPositionalArgument(QStringLiteral("config"), i18n("Modify the Baloo configuration"));
     parser.addPositionalArgument(QStringLiteral("monitor"), i18n("Monitor the file indexer"));
     parser.addPositionalArgument(QStringLiteral("indexSize"), i18n("Display the disk space used by index"));
-    parser.addOption(QCommandLineOption(QStringList() << QStringLiteral("f") << QStringLiteral("format"), 
-        i18n("Output format <%1|%2|%3>.\nOnly applies to \"%4\"", 
-             QStringLiteral("json"), 
-             QStringLiteral("simple"), 
-             QStringLiteral("multiline (default)"),
-             QStringLiteral("balooctl status <file>")
-            )
-        , i18n("format")
-        
-        , QStringLiteral("multiline")));
+    parser.addPositionalArgument(QStringLiteral("failed"), i18n("Display files which could not be indexed"));
+
+    QString statusFormatDescription = i18nc("Format to use for status command, %1|%2|%3 are option values, %4 is a CLI command",
+                                            "Output format <%1|%2|%3>.\nThe default format is \"%1\".\nOnly applies to \"%4\"",
+                                                QStringLiteral("multiline"),
+                                                QStringLiteral("json"),
+                                                QStringLiteral("simple"),
+                                                QStringLiteral("balooctl status <file>"));
+    parser.addOption({{QStringLiteral("f"), QStringLiteral("format")},
+                     statusFormatDescription, i18n("format"), QStringLiteral("multiline")});
+
     parser.addVersionOption();
     parser.addHelpOption();
 
@@ -143,39 +141,52 @@ int main(int argc, char* argv[])
         cfg.setFileIndexingEnabled(isEnabled);
 
         if (isEnabled) {
-            out << "Enabling the File Indexer\n";
-
-            cfg.setFirstRun(true);
-            start();
+            bool running = mainInterface.isValid();
+            if (running) {
+                out << "File Indexer already running\n";
+            } else {
+                out << "Enabling and starting the File Indexer\n";
+                start();
+            }
         } else {
-            out << "Disabling the File Indexer\n";
+            out << "Disabling and stopping the File Indexer\n";
 
             mainInterface.quit();
-            const QString path = fileIndexDbPath() + QStringLiteral("/index");
-            QFile(path).remove();
         }
 
         return 0;
     }
 
-    if (command == QLatin1String("start") || command == QLatin1String("stop") ||
-        command == QLatin1String("restart")) {
-        bool shouldStart = false;
-        bool shouldStop = false;
+    if (command == QLatin1String("purge")) {
+        bool running = mainInterface.isValid();
 
-        if (command == QLatin1String("start"))
-            shouldStart = true;
-        else if (command == QLatin1String("stop"))
-            shouldStop = true;
-        else if (command == QLatin1String("restart")) {
-            shouldStart = true;
-            shouldStop = true;
+        if (running) {
+            mainInterface.quit();
+            out << "Stopping the File Indexer ...";
+            for (int i = 5 * 60; i; --i) {
+                QCoreApplication::processEvents();
+                if (!mainInterface.isValid()) {
+                    break;
+                }
+                out << "." << flush;
+                QThread::msleep(200);
+            }
+            if (!mainInterface.isValid()) {
+                out << " - done\n";
+            } else {
+                out << " - failed to stop!\n";
+                return 1;
+            }
         }
 
-        if (shouldStop)
-            mainInterface.quit();
-        if (shouldStart)
+        const QString path = fileIndexDbPath() + QStringLiteral("/index");
+        QFile(path).remove();
+        out << "Deleted the index database\n";
+
+        if (running) {
             start();
+            out << "Restarting the File Indexer\n";
+        }
 
         return 0;
     }
@@ -258,11 +269,6 @@ int main(int argc, char* argv[])
                 out << "Could not stat file: " << url << endl;
                 continue;
             }
-            if (tr.documentData(id).isEmpty()) {
-                out << "Skipping: " << url << " Reason: Not yet indexed\n";
-                continue;
-            }
-            Indexer indexer(url, &tr);
             out << "Clearing " << url << endl;
             tr.removeDocument(id);
         }
@@ -272,6 +278,32 @@ int main(int argc, char* argv[])
         return 0;
     }
 
+    if (command == QStringLiteral("failed")) {
+        Database *db = globalDatabaseInstance();
+        if (!db->open(Database::ReadOnlyDatabase)) {
+            out << "Baloo Index could not be opened\n";
+            return 1;
+        }
+
+        Transaction tr(db, Transaction::ReadOnly);
+
+        const quint64 limit = 128;
+        const QVector<quint64> failedIds = tr.failedIds(limit);
+        if (failedIds.isEmpty()) {
+            out << "All Files were indexed successfully\n";
+            return 0;
+        }
+
+        out << "The folling files could not be indexed:\n";
+        for (auto id : failedIds) {
+            out << tr.documentUrl(id) << '\n';
+        }
+        if (failedIds.size() == limit) {
+            out << "... list truncated\n";
+        }
+        return 0;
+    }
+
     if (command == QStringLiteral("indexSize")) {
         Database *db = globalDatabaseInstance();
         if (!db->open(Database::ReadOnlyDatabase)) {
@@ -284,9 +316,10 @@ int main(int argc, char* argv[])
             Transaction tr(db, Transaction::ReadOnly);
             size = tr.dbSize();
         }
+        uint totalDataSize = size.expectedSize;
 
         KFormat format(QLocale::system());
-        auto prFunc = [&](const QString& name, uint size, uint totalSize) {
+        auto prFunc = [&](const QString& name, uint size) {
             out.setFieldWidth(20);
             out << name;
             out.setFieldWidth(0);
@@ -294,26 +327,25 @@ int main(int argc, char* argv[])
             out.setFieldWidth(15);
             out << format.formatByteSize(size, 2);
             out.setFieldWidth(10);
-            out << QString::number((100.0 * size / totalSize), 'f', 3);
+            out << QString::number((100.0 * size / totalDataSize), 'f', 3);
             out.setFieldWidth(0);
             out << " %\n";
         };
 
-        uint ts = size.expectedSize;
-        out << "Actual Size: " << format.formatByteSize(size.actualSize, 2) << "\n";
-        out << "Expected Size: " << format.formatByteSize(size.expectedSize, 2) << "\n\n";
-        prFunc(QStringLiteral("PostingDB"), size.postingDb, ts);
-        prFunc(QStringLiteral("PosistionDB"), size.positionDb, ts);
-        prFunc(QStringLiteral("DocTerms"), size.docTerms, ts);
-        prFunc(QStringLiteral("DocFilenameTerms"), size.docFilenameTerms, ts);
-        prFunc(QStringLiteral("DocXattrTerms"), size.docXattrTerms, ts);
-        prFunc(QStringLiteral("IdTree"), size.idTree, ts);
-        prFunc(QStringLiteral("IdFileName"), size.idFilename, ts);
-        prFunc(QStringLiteral("DocTime"), size.docTime, ts);
-        prFunc(QStringLiteral("DocData"), size.docData, ts);
-        prFunc(QStringLiteral("ContentIndexingDB"), size.contentIndexingIds, ts);
-        prFunc(QStringLiteral("FailedIdsDB"), size.failedIds, ts);
-        prFunc(QStringLiteral("MTimeDB"), size.mtimeDb, ts);
+        out << "File Size: " << format.formatByteSize(size.actualSize, 2) << "\n";
+        out << "Used:      " << format.formatByteSize(totalDataSize, 2) << "\n\n";
+        prFunc(QStringLiteral("PostingDB"), size.postingDb);
+        prFunc(QStringLiteral("PositionDB"), size.positionDb);
+        prFunc(QStringLiteral("DocTerms"), size.docTerms);
+        prFunc(QStringLiteral("DocFilenameTerms"), size.docFilenameTerms);
+        prFunc(QStringLiteral("DocXattrTerms"), size.docXattrTerms);
+        prFunc(QStringLiteral("IdTree"), size.idTree);
+        prFunc(QStringLiteral("IdFileName"), size.idFilename);
+        prFunc(QStringLiteral("DocTime"), size.docTime);
+        prFunc(QStringLiteral("DocData"), size.docData);
+        prFunc(QStringLiteral("ContentIndexingDB"), size.contentIndexingIds);
+        prFunc(QStringLiteral("FailedIdsDB"), size.failedIds);
+        prFunc(QStringLiteral("MTimeDB"), size.mtimeDb);
 
         return 0;
     }
@@ -323,7 +355,9 @@ int main(int argc, char* argv[])
         return mon.exec(parser);
     }
 
-    if (command == QStringLiteral("checkDb")) {
+    /*
+     TODO: Make separate executable
+     if (command == QStringLiteral("checkDb")) {
         Database *db = globalDatabaseInstance();
         if (!db->open(Database::ReadOnlyDatabase)) {
             out << "Baloo Index could not be opened\n";
@@ -337,6 +371,7 @@ int main(int argc, char* argv[])
         tr.checkFsTree();
         return 0;
     }
+    */
 
     parser.showHelp(1);
     return 0;
